from pylab import *
rc('font', size=12)

dset_names = dict(
    total='Total time',
    setup='Setup time',
    solve='Solve time',
    iters='Iterations'
    )

handles = []

figure(figsize=(8,8))
gs = GridSpec(2,2)

def set_ticks(ax, t):
    ax.set_xticks(t)
    ax.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())
    ax.get_xaxis().set_tick_params(which='minor', size=0)
    ax.get_xaxis().set_tick_params(which='minor', width=0)

for test in ('amgcl', 'amgcl-schur', 'petsc', 'trilinos'):
    data = loadtxt('smem_data/nstokes/{}.txt'.format(test), delimiter=' ', dtype=dict(
        names=('np', 'size', 'iters', 'setup', 'solve'),
        formats=('i4', 'i4', 'i4', 'f8', 'f8'))
    )

    np = data['np']
    cores = unique(sorted(data['np']))
    setup = empty(cores.shape)
    solve = empty(cores.shape)
    iters = empty(cores.shape)

    for i,n in enumerate(cores):
        setup[i] = median(data[np==n]['setup'])
        solve[i] = median(data[np==n]['solve'])
        iters[i] = median(data[np==n]['iters'])

    total = setup + solve

    for r,dset in enumerate(('total', 'setup', 'solve', 'iters')):
        subplot(gs[r])

        if dset == 'iters':
            h = semilogx(cores, eval(dset), marker='o')
        else:
            h = loglog(cores, eval(dset), marker='o')
        set_ticks(gca(), [1, 2, 4, 8, 16])
        ylabel(dset_names[dset])

        if r == 0: handles.append(h[0])
        if r >= 2: xlabel('Cores/MPI processes')


for test in ('amgcl-vexcl-cuda', 'amgcl-schur-cuda'):
    data = loadtxt('smem_data/nstokes/{}.txt'.format(test), delimiter=' ', dtype=dict(
        names=('np', 'size', 'iters', 'setup', 'solve'),
        formats=('i4', 'i4', 'i4', 'f8', 'f8')))

    total = ones_like(cores) * median(data['setup'] + data['solve'])
    setup = ones_like(cores) * median(data['setup'])
    solve = ones_like(cores) * median(data['solve'])
    iters = ones_like(cores) * median(data['iters'])

    for r,dset in enumerate(('total', 'setup', 'solve', 'iters')):
        subplot(gs[r])
        h = plot(cores, eval(dset), '--')
        if r == 0: handles.append(h[0])

tight_layout()

figlegend(handles, [
    'AMGCL (block)', 'AMGCL (split)', 'PETSC (split)', 'Trilinos (block)',
    'AMGCL (block, VexCL)', 'AMGCL (split, CUDA)'
    ],
    ncol=3, loc='lower center')
gcf().suptitle('3D Navier-Stokes problem')
gcf().subplots_adjust(top=0.93, bottom=0.15)

show()