import os
from pylab import *
rc('font', size=12)

def load_data(fname):
    return loadtxt(fname, dtype={
        'names'   : ('size', 'omp', 'mpi', 'setup', 'solve', 'iters'),
        'formats' : ('i8', 'i4', 'i4', 'f8', 'f8', 'i4')
        })

def set_ticks(ax, t):
    ax.set_xscale('log')
    ax.set_xticks(t[0::2])
    ax.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())
    ax.get_xaxis().set_tick_params(which='minor', size=0)
    ax.get_xaxis().set_tick_params(which='minor', width=0)

figure(figsize=(8,10))
gs = GridSpec(4,2)
handles = []

for k,fname in (
        (0, 'dmem_data/daint/gpu_linear_weak.dat'),
        (0, 'dmem_data/daint/cpu_linear_weak.dat'),
        (1, 'dmem_data/daint/gpu_const_weak.dat'),
        (1, 'dmem_data/daint/cpu_const_weak.dat'),
        ):
    d = load_data(fname)
    m = unique(d['mpi'])

    setup = array([min(d[d['mpi']==i]['setup']) for i in m])
    solve = array([min(d[d['mpi']==i]['solve']) for i in m])
    iters = array([min(d[d['mpi']==i]['iters']) for i in m])
    total = setup + solve

    ax = subplot(gs[0,k])
    h = loglog(m, total, '.-')
    ylim([1e0,200])
    set_ticks(ax, m)
    if k == 0: handles.append(h[0])

    ax = subplot(gs[1,k])
    loglog(m, setup, '.-')
    ylim([1e0,20])
    set_ticks(ax, m)

    ax = subplot(gs[2,k])
    loglog(m, solve, '.-')
    ylim([1e0,200])
    set_ticks(ax, m)

    ax = subplot(gs[3,k])
    semilogx(m, iters, '.-')
    ylim([0,160])
    set_ticks(ax, m)
    xlabel('Compute nodes')

for fname in ('dmem_data/daint/trilinos_weak.dat',):
    tri = loadtxt(f'{os.path.dirname(sys.argv[0])}/{fname}', dtype={
        'names'   : ('mpi', 'size', 'iters', 'setup', 'solve'),
        'formats' : ('i4', 'i8', 'i4', 'f8', 'f8')
        })

    for k in (0,1):
        subplot(gs[0,k])
        h = plot(tri['mpi']//12, tri['setup'] + tri['solve'], '.-')
        if k == 0: handles += h

        subplot(gs[1,k])
        plot(tri['mpi']//12, tri['setup'], '.-')

        subplot(gs[2,k])
        plot(tri['mpi']//12, tri['solve'], '.-')

        subplot(gs[3,k])
        plot(tri['mpi']//12, tri['iters'], '.-')

subplot(gs[0,0])
title('Linear deflation')
ylabel('Total time')

subplot(gs[0,1])
title('Constant deflation')

subplot(gs[1,0])
ylabel('Setup time')

subplot(gs[2,0])
ylabel('Solve time')

subplot(gs[3,0])
ylabel('Iterations')

tight_layout()

figlegend(handles, ('GPU', 'CPU (OMP=12)', 'Trilinos'), ncol=3, loc='lower center')
gcf().suptitle('Weak scaling of the Poisson problem on PizDaint cluster')
gcf().subplots_adjust(top=0.93, bottom=0.1)

show()